@@ -32,6 +32,7 @@
 #include <spa/utils/list.h>
 #include <spa/utils/names.h>
 #include <spa/utils/json.h>
+#include <spa/utils/string.h>
 #include <spa/node/keys.h>
 #include <spa/node/node.h>
 #include <spa/node/io.h>
@@ -143,6 +144,8 @@ struct impl {
 	struct spa_log *log;
 	struct spa_cpu *cpu;
 
+	struct spa_io_position *io_position;
+
 	struct spa_hook_list hooks;
 
 	uint64_t info_all;
@@ -177,11 +180,12 @@ struct impl {
 
 static void emit_info(struct impl *this, bool full)
 {
+	uint64_t old = full ? this->info.change_mask : 0;
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
 		spa_node_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
@@ -376,7 +380,7 @@ static int impl_node_enum_params(void *object, int seq,
 	struct impl *this = object;
 	struct spa_pod *param;
 	struct spa_pod_builder b = { 0 };
-	uint8_t buffer[1024];
+	uint8_t buffer[4096];
 	struct spa_result_node_params result;
 	uint32_t count = 0;
 
@@ -512,6 +516,9 @@ static int apply_props(struct impl *this, const struct spa_pod *param)
 	bool have_channel_volume = false;
 	bool have_soft_volume = false;
 
+	if (param == NULL)
+		return 0;
+
 	SPA_POD_OBJECT_FOREACH(obj, prop) {
 		switch (prop->key) {
 		case SPA_PROP_volume:
@@ -593,7 +600,20 @@ static int apply_midi(struct impl *this, const struct spa_pod *value)
 
 static int impl_node_set_io(void *object, uint32_t id, void *data, size_t size)
 {
-	return -ENOTSUP;
+	struct impl *this = object;
+
+	spa_return_val_if_fail(this != NULL, -EINVAL);
+
+	spa_log_debug(this->log, NAME " %p: io %d %p/%zd", this, id, data, size);
+
+	switch (id) {
+	case SPA_IO_Position:
+		this->io_position = data;
+		break;
+	default:
+		return -ENOENT;
+	}
+	return 0;
 }
 
 static int impl_node_set_param(void *object, uint32_t id, uint32_t flags,
@@ -638,12 +658,13 @@ static int impl_node_send_command(void *object, const struct spa_command *comman
 
 static void emit_port_info(struct impl *this, struct port *port, bool full)
 {
+	uint64_t old = full ? port->info.change_mask : 0;
 	if (full)
 		port->info.change_mask = port->info_all;
 	if (port->info.change_mask) {
 		spa_node_emit_port_info(&this->hooks,
 				port->direction, port->id, &port->info);
-		port->info.change_mask = 0;
+		port->info.change_mask = old;
 	}
 }
 
@@ -726,18 +747,23 @@ static int port_enum_formats(void *object,
 				SPA_FORMAT_mediaSubtype,   SPA_POD_Id(SPA_MEDIA_SUBTYPE_raw),
 				SPA_FORMAT_AUDIO_format,   SPA_POD_Id(SPA_AUDIO_FORMAT_F32P),
 				0);
+
 			if (other->have_format) {
 				spa_pod_builder_add(builder,
 					SPA_FORMAT_AUDIO_rate, SPA_POD_Int(other->format.info.raw.rate),
+					SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(
+						other->format.info.raw.channels, 1, INT32_MAX),
 					0);
 			} else {
+				uint32_t rate = this->io_position ?
+					this->io_position->clock.rate.denom : DEFAULT_RATE;
+
 				spa_pod_builder_add(builder,
-					SPA_FORMAT_AUDIO_rate, SPA_POD_CHOICE_RANGE_Int(DEFAULT_RATE, 1, INT32_MAX),
+					SPA_FORMAT_AUDIO_rate, SPA_POD_CHOICE_RANGE_Int(rate, 0, INT32_MAX),
+					SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(
+						DEFAULT_CHANNELS, 1, INT32_MAX),
 					0);
 			}
-			spa_pod_builder_add(builder,
-				SPA_FORMAT_AUDIO_channels, SPA_POD_CHOICE_RANGE_Int(DEFAULT_CHANNELS, 1, INT32_MAX),
-				0);
 			*param = spa_pod_builder_pop(builder, &f);
 		}
 		break;
@@ -1312,7 +1338,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Node))
 		*interface = &this->node;
 	else
 		return -ENOENT;
@@ -1336,7 +1362,7 @@ static uint32_t channel_from_name(const char *name)
 {
 	int i;
 	for (i = 0; spa_type_audio_channel[i].name; i++) {
-		if (strcmp(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)) == 0)
+		if (spa_streq(name, spa_debug_type_short_name(spa_type_audio_channel[i].name)))
 			return spa_type_audio_channel[i].type;
 	}
 	return SPA_AUDIO_CHANNEL_UNKNOWN;
@@ -1391,18 +1417,15 @@ impl_init(const struct spa_handle_factory *factory,
 	for (i = 0; info && i < info->n_items; i++) {
 		const char *k = info->items[i].key;
 		const char *s = info->items[i].value;
-		if (strcmp(k, "channelmix.normalize") == 0 &&
-		    (strcmp(s, "true") == 0 || atoi(s) != 0))
+		if (spa_streq(k, "channelmix.normalize") && spa_atob(s))
 			this->mix.options |= CHANNELMIX_OPTION_NORMALIZE;
-		if (strcmp(k, "channelmix.mix-lfe") == 0 &&
-		    (strcmp(s, "true") == 0 || atoi(s) != 0))
+		if (spa_streq(k, "channelmix.mix-lfe") && spa_atob(s))
 			this->mix.options |= CHANNELMIX_OPTION_MIX_LFE;
-		if (strcmp(k, "channelmix.upmix") == 0 &&
-		    (strcmp(s, "true") == 0 || atoi(s) != 0))
+		if (spa_streq(k, "channelmix.upmix") && spa_atob(s))
 			this->mix.options |= CHANNELMIX_OPTION_UPMIX;
-		if (strcmp(k, "channelmix.lfe-cutoff") == 0)
+		if (spa_streq(k, "channelmix.lfe-cutoff"))
 			this->mix.lfe_cutoff = atoi(s);
-		if (strcmp(k, SPA_KEY_AUDIO_POSITION) == 0)
+		if (spa_streq(k, SPA_KEY_AUDIO_POSITION))
 			this->props.n_channels = parse_position(this->props.channel_map, s, strlen(s));
 	}
 	this->props.channel.n_volumes = this->props.n_channels;
