@@ -33,9 +33,7 @@
  * </refsect2>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 #include "gstpipewiresrc.h"
 #include "gstpipewireformat.h"
 
@@ -409,7 +407,7 @@ buffer_recycle (GstMiniObject *obj)
   src = data->owner;
   data->queued = TRUE;
 
-  GST_LOG_OBJECT (obj, "recycle buffer");
+  GST_LOG_OBJECT (src, "recycle buffer %p", obj);
   pw_thread_loop_lock (src->core->loop);
   if (src->stream)
     pw_stream_queue_buffer (src->stream, data->b);
@@ -787,7 +785,11 @@ on_param_changed (void *data, uint32_t id,
         SPA_PARAM_BUFFERS_blocks,  SPA_POD_CHOICE_RANGE_Int(0, 1, INT32_MAX),
         SPA_PARAM_BUFFERS_size,    SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
         SPA_PARAM_BUFFERS_stride,  SPA_POD_CHOICE_RANGE_Int(0, 0, INT32_MAX),
-        SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16));
+        SPA_PARAM_BUFFERS_align,   SPA_POD_Int(16),
+        SPA_PARAM_BUFFERS_dataType, SPA_POD_CHOICE_FLAGS_Int(
+						(1<<SPA_DATA_DmaBuf) |
+						(1<<SPA_DATA_MemFd) |
+						(1<<SPA_DATA_MemPtr)));
 
     params[1] = spa_pod_builder_add_object (&b,
         SPA_TYPE_OBJECT_ParamMeta, SPA_PARAM_Meta,
