@@ -22,6 +22,22 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include <arpa/inet.h>
+#include <math.h>
+
+#include <spa/debug/buffer.h>
+#include <spa/utils/defs.h>
+#include <spa/utils/string.h>
+#include <pipewire/keys.h>
+#include <pipewire/log.h>
+
+#include "defs.h"
+#include "format.h"
+#include "internal.h"
+#include "media-roles.h"
+#include "message.h"
+#include "volume.h"
+
 #define VOLUME_MUTED ((uint32_t) 0U)
 #define VOLUME_NORM ((uint32_t) 0x10000U)
 #define VOLUME_MAX ((uint32_t) UINT32_MAX/2)
@@ -45,26 +61,7 @@ static inline float volume_to_linear(uint32_t vol)
 	return v * v * v;
 }
 
-struct str_map {
-	const char *pw_str;
-	const char *pa_str;
-	const struct str_map *child;
-};
-
-const struct str_map media_role_map[] = {
-	{ "Movie", "video", },
-	{ "Music", "music", },
-	{ "Game", "game", },
-	{ "Notification", "event", },
-	{ "Communication", "phone", },
-	{ "Movie", "animation", },
-	{ "Production", "production", },
-	{ "Accessibility", "a11y", },
-	{ "Test", "test", },
-	{ NULL, NULL },
-};
-
-const struct str_map key_table[] = {
+static const struct str_map key_table[] = {
 	{ PW_KEY_DEVICE_BUS_PATH, "device.bus_path" },
 	{ PW_KEY_DEVICE_FORM_FACTOR, "device.form_factor" },
 	{ PW_KEY_DEVICE_ICON_NAME, "device.icon_name" },
@@ -78,51 +75,6 @@ const struct str_map key_table[] = {
 	{ NULL, NULL },
 };
 
-static inline const struct str_map *str_map_find(const struct str_map *map, const char *pw, const char *pa)
-{
-	uint32_t i;
-	for (i = 0; map[i].pw_str; i++)
-		if ((pw && strcmp(map[i].pw_str, pw) == 0) ||
-		    (pa && strcmp(map[i].pa_str, pa) == 0))
-			return &map[i];
-	return NULL;
-}
-
-enum {
-	TAG_INVALID = 0,
-	TAG_STRING = 't',
-	TAG_STRING_NULL = 'N',
-	TAG_U32 = 'L',
-	TAG_U8 = 'B',
-	TAG_U64 = 'R',
-	TAG_S64 = 'r',
-	TAG_SAMPLE_SPEC = 'a',
-	TAG_ARBITRARY = 'x',
-	TAG_BOOLEAN_TRUE = '1',
-	TAG_BOOLEAN_FALSE = '0',
-	TAG_BOOLEAN = TAG_BOOLEAN_TRUE,
-	TAG_TIMEVAL = 'T',
-	TAG_USEC = 'U'  /* 64bit unsigned */,
-	TAG_CHANNEL_MAP = 'm',
-	TAG_CVOLUME = 'v',
-	TAG_PROPLIST = 'P',
-	TAG_VOLUME = 'V',
-	TAG_FORMAT_INFO = 'f',
-};
-
-struct message {
-	struct spa_list link;
-	struct stats *stat;
-	uint32_t extra[4];
-	uint32_t channel;
-	uint32_t allocated;
-	uint32_t length;
-	uint32_t offset;
-	uint8_t *data;
-};
-
-static int message_get(struct message *m, ...);
-
 static int read_u8(struct message *m, uint8_t *val)
 {
 	if (m->offset + 1 > m->length)
@@ -226,10 +178,10 @@ static int read_arbitrary(struct message *m, const void **val, size_t *length)
 static int read_string(struct message *m, char **str)
 {
 	uint32_t n, maxlen = m->length - m->offset;
-	n = strnlen(SPA_MEMBER(m->data, m->offset, char), maxlen);
+	n = strnlen(SPA_PTROFF(m->data, m->offset, char), maxlen);
 	if (n == maxlen)
 		return -EINVAL;
-	*str = SPA_MEMBER(m->data, m->offset, char);
+	*str = SPA_PTROFF(m->data, m->offset, char);
 	m->offset += n + 1;
 	return 0;
 }
@@ -317,7 +269,7 @@ static int read_format_info(struct message *m, struct format_info *info)
 	return res;
 }
 
-static int message_get(struct message *m, ...)
+int message_get(struct message *m, ...)
 {
 	va_list va;
 	int res = 0;
@@ -481,7 +433,7 @@ static void write_string(struct message *m, const char *s)
 	if (s != NULL) {
 		int len = strlen(s) + 1;
 		if (ensure_size(m, len) > 0)
-			strcpy(SPA_MEMBER(m->data, m->length, char), s);
+			strcpy(SPA_PTROFF(m->data, m->length, char), s);
 		m->length += len;
 	}
 }
@@ -563,34 +515,34 @@ static void write_cvolume(struct message *m, struct volume *vol)
 static void add_stream_group(struct message *m, struct spa_dict *dict, const char *key,
 		const char *media_class, const char *media_role)
 {
-	const char *str, *fmt, *prefix;
+	const char *str, *id, *prefix;
 	char *b;
 	int l;
 
 	if (media_class == NULL)
 		return;
-	if (strcmp(media_class, "Stream/Output/Audio") == 0)
+	if (spa_streq(media_class, "Stream/Output/Audio"))
 		prefix = "sink-input";
-	else if (strcmp(media_class, "Stream/Input/Audio") == 0)
+	else if (spa_streq(media_class, "Stream/Input/Audio"))
 		prefix = "source-output";
 	else
 		return;
 
 	if ((str = media_role) != NULL)
-		fmt = "%s-by-media-role:%s";
+		id = "media-role";
 	else if ((str = spa_dict_lookup(dict, PW_KEY_APP_ID)) != NULL)
-		fmt = "%s-by-application-id:%s";
+		id = "application-id";
 	else if ((str = spa_dict_lookup(dict, PW_KEY_APP_NAME)) != NULL)
-		fmt = "%s-by-application-name:%s";
+		id = "application-name";
 	else if ((str = spa_dict_lookup(dict, PW_KEY_MEDIA_NAME)) != NULL)
-		fmt = "%s-by-media-name:%s";
+		id = "media-name";
 	else
 		return;
 
 	write_string(m, key);
-	l = strlen(fmt) + strlen(prefix) + strlen(str) - 3;
+	l = strlen(prefix) + strlen(id) + strlen(str) + 6; /* "-by-" , ":" and \0 */
 	b = alloca(l);
-	snprintf(b, l, fmt, prefix, str);
+	snprintf(b, l, "%s-by-%s:%s", prefix, id, str);
 	write_u32(m, l);
 	write_arbitrary(m, b, l);
 }
@@ -614,9 +566,9 @@ static void write_dict(struct message *m, struct spa_dict *dict, bool remap)
 				    (map = str_map_find(map->child, val, NULL)) != NULL)
 					val = map->pa_str;
 			}
-			if (strcmp(key, "media.class") == 0)
+			if (spa_streq(key, "media.class"))
 				media_class = val;
-			if (strcmp(key, "media.role") == 0)
+			if (spa_streq(key, "media.role"))
 				media_role = val;
 
 			write_string(m, key);
@@ -639,7 +591,7 @@ static void write_format_info(struct message *m, struct format_info *info)
 	write_dict(m, info->props ? &info->props->dict : NULL, false);
 }
 
-static int message_put(struct message *m, ...)
+int message_put(struct message *m, ...)
 {
 	va_list va;
 
@@ -709,7 +661,7 @@ static int message_put(struct message *m, ...)
 	return 0;
 }
 
-static int message_dump(enum spa_log_level level, struct message *m)
+int message_dump(enum spa_log_level level, struct message *m)
 {
 	int res;
 	uint32_t i, offset = m->offset, o;
@@ -869,3 +821,51 @@ static int message_dump(enum spa_log_level level, struct message *m)
 
 	return 0;
 }
+
+struct message *message_alloc(struct impl *impl, uint32_t channel, uint32_t size)
+{
+	struct message *msg;
+
+	if (!spa_list_is_empty(&impl->free_messages)) {
+		msg = spa_list_first(&impl->free_messages, struct message, link);
+		spa_list_remove(&msg->link);
+		pw_log_trace("using recycled message %p", msg);
+	} else {
+		if ((msg = calloc(1, sizeof(*msg))) == NULL)
+			return NULL;
+
+		pw_log_trace("new message %p", msg);
+		msg->stat = &impl->stat;
+		msg->stat->n_allocated++;
+		msg->stat->n_accumulated++;
+	}
+
+	if (ensure_size(msg, size) < 0) {
+		message_free(impl, msg, false, true);
+		return NULL;
+	}
+
+	spa_zero(msg->extra);
+	msg->channel = channel;
+	msg->offset = 0;
+	msg->length = size;
+
+	return msg;
+}
+
+void message_free(struct impl *impl, struct message *msg, bool dequeue, bool destroy)
+{
+	if (dequeue)
+		spa_list_remove(&msg->link);
+
+	if (destroy) {
+		pw_log_trace("destroy message %p", msg);
+		msg->stat->n_allocated--;
+		msg->stat->allocated -= msg->allocated;
+		free(msg->data);
+		free(msg);
+	} else {
+		pw_log_trace("recycle message %p", msg);
+		spa_list_append(&impl->free_messages, &msg->link);
+	}
+}
