@@ -41,6 +41,7 @@
 #endif
 
 #include <spa/utils/result.h>
+#include <spa/utils/string.h>
 #include <spa/utils/json.h>
 
 #include <pipewire/impl.h>
@@ -63,12 +64,20 @@ static int make_path(char *path, int size, const char *paths[])
 	return 0;
 }
 
-static int get_read_path(char *path, size_t size, const char *prefix, const char *name)
+static int get_config_path(char *path, size_t size, const char *prefix, const char *name)
 {
 	const char *dir;
 	char buffer[4096];
 
-	if (prefix[0] == '/') {
+	if (name[0] == '/') {
+		const char *paths[] = { name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+		return -ENOENT;
+	}
+
+	if (prefix && prefix[0] == '/') {
 		const char *paths[] = { prefix, name, NULL };
 		if (make_path(path, size, paths) == 0 &&
 		    access(path, R_OK) == 0)
@@ -76,6 +85,22 @@ static int get_read_path(char *path, size_t size, const char *prefix, const char
 		return -ENOENT;
 	}
 
+	if (prefix == NULL) {
+		prefix = name;
+		name = NULL;
+	}
+
+	if (pw_check_option("no-config", "true"))
+		goto no_config;
+
+	dir = getenv("PIPEWIRE_CONFIG_DIR");
+	if (dir != NULL) {
+		const char *paths[] = { dir, prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+
 	dir = getenv("XDG_CONFIG_HOME");
 	if (dir != NULL) {
 		const char *paths[] = { dir, "pipewire", prefix, name, NULL };
@@ -95,9 +120,16 @@ static int get_read_path(char *path, size_t size, const char *prefix, const char
 		    access(path, R_OK) == 0)
 			return 1;
 	}
-	dir = getenv("PIPEWIRE_CONFIG_DIR");
-	if (dir == NULL)
-		dir = PIPEWIRE_CONFIG_DIR;
+
+	dir = PIPEWIRE_CONFIG_DIR;
+	if (dir != NULL) {
+		const char *paths[] = { dir, prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+no_config:
+	dir = PIPEWIRE_CONFDATADIR;
 	if (dir != NULL) {
 		const char *paths[] = { dir, prefix, name, NULL };
 		if (make_path(path, size, paths) == 0 &&
@@ -107,9 +139,73 @@ static int get_read_path(char *path, size_t size, const char *prefix, const char
 	return 0;
 }
 
+static int get_state_path(char *path, size_t size, const char *prefix, const char *name)
+{
+	const char *dir;
+	char buffer[4096];
+
+	if (name[0] == '/') {
+		const char *paths[] = { name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+		return -ENOENT;
+	}
+
+	if (prefix && prefix[0] == '/') {
+		const char *paths[] = { prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+		return -ENOENT;
+	}
+
+	if (prefix == NULL) {
+		prefix = name;
+		name = NULL;
+	}
+
+	dir = getenv("PIPEWIRE_STATE_DIR");
+	if (dir != NULL) {
+		const char *paths[] = { dir, prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+
+	dir = getenv("XDG_STATE_HOME");
+	if (dir != NULL) {
+		const char *paths[] = { dir, "pipewire", prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+	dir = getenv("HOME");
+	if (dir == NULL) {
+		struct passwd pwd, *result = NULL;
+		if (getpwuid_r(getuid(), &pwd, buffer, sizeof(buffer), &result) == 0)
+			dir = result ? result->pw_dir : NULL;
+	}
+	if (dir != NULL) {
+		const char *paths[] = { dir, ".local", "state", "pipewire", prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+	if (dir != NULL) {
+		/* fallback for old XDG_CONFIG_HOME */
+		const char *paths[] = { dir, ".config", "pipewire", prefix, name, NULL };
+		if (make_path(path, size, paths) == 0 &&
+		    access(path, R_OK) == 0)
+			return 1;
+	}
+
+	return 0;
+}
+
 static int ensure_path(char *path, int size, const char *paths[])
 {
-	int i, len, res, mode;
+	int i, len, mode;
 	char *p = path;
 
 	for (i = 0; paths[i] != NULL; i++) {
@@ -126,14 +222,14 @@ static int ensure_path(char *path, int size, const char *paths[])
 		if (paths[i+1] == NULL)
 			mode |= R_OK | W_OK;
 
-		if ((res = access(path, mode)) < 0) {
+		if (access(path, mode) < 0) {
 			if (errno != ENOENT)
 				return -errno;
-			if ((res = mkdir(path, 0700)) < 0) {
+			if (mkdir(path, 0700) < 0) {
 				pw_log_info("Can't create directory %s: %m", path);
                                 return -errno;
 			}
-			if ((res = access(path, mode)) < 0)
+			if (access(path, mode) < 0)
 				return -errno;
 
 			pw_log_info("created directory %s", path);
@@ -153,7 +249,7 @@ static int open_write_dir(char *path, int size, const char *prefix)
 		if (ensure_path(path, size, paths) == 0)
 			goto found;
 	}
-	dir = getenv("XDG_CONFIG_HOME");
+	dir = getenv("XDG_STATE_HOME");
 	if (dir != NULL) {
 		const char *paths[] = { dir, "pipewire", prefix, NULL };
 		if (ensure_path(path, size, paths) == 0)
@@ -166,7 +262,7 @@ static int open_write_dir(char *path, int size, const char *prefix)
 			dir = result ? result->pw_dir : NULL;
 	}
 	if (dir != NULL) {
-		const char *paths[] = { dir, ".config", "pipewire", prefix, NULL };
+		const char *paths[] = { dir, ".local", "state", "pipewire", prefix, NULL };
 		if (ensure_path(path, size, paths) == 0)
 			goto found;
 	}
@@ -182,10 +278,9 @@ found:
 SPA_EXPORT
 int pw_conf_save_state(const char *prefix, const char *name, struct pw_properties *conf)
 {
-	const struct spa_dict_item *it;
 	char path[PATH_MAX];
 	char *tmp_name;
-	int res, sfd, fd;
+	int res, sfd, fd, count = 0;
 	FILE *f;
 
 	if ((sfd = open_write_dir(path, sizeof(path), prefix)) < 0)
@@ -193,23 +288,16 @@ int pw_conf_save_state(const char *prefix, const char *name, struct pw_propertie
 
 	tmp_name = alloca(strlen(name)+5);
 	sprintf(tmp_name, "%s.tmp", name);
-	if ((fd = openat(sfd, tmp_name,  O_CLOEXEC | O_CREAT | O_WRONLY | O_TRUNC, 0700)) < 0) {
+	if ((fd = openat(sfd, tmp_name,  O_CLOEXEC | O_CREAT | O_WRONLY | O_TRUNC, 0600)) < 0) {
 		pw_log_error("can't open file '%s': %m", tmp_name);
 		res = -errno;
 		goto error;
 	}
 
 	f = fdopen(fd, "w");
-	fprintf(f, "{ \n");
-	spa_dict_for_each(it, &conf->dict) {
-		char key[1024];
-
-		if (spa_json_encode_string(key, sizeof(key)-1, it->key) >= (int)sizeof(key)-1)
-			continue;
-
-		fprintf(f, " %s: %s\n", key, it->value);
-	}
-	fprintf(f, "}\n");
+	fprintf(f, "{");
+	count += pw_properties_serialize_dict(f, &conf->dict, PW_PROPERTIES_FLAG_NL);
+	fprintf(f, "%s}", count == 0 ? " " : "\n");
 	fclose(f);
 
 	if (renameat(sfd, tmp_name, sfd, name) < 0) {
@@ -224,21 +312,12 @@ error:
 	return res;
 }
 
-static int conf_load(const char *prefix, const char *name, struct pw_properties *conf)
+static int conf_load(const char *path, struct pw_properties *conf)
 {
-	char path[PATH_MAX], *data;
+	char *data;
 	struct stat sbuf;
 	int fd;
 
-	if (prefix == NULL) {
-		prefix = name;
-		name = NULL;
-	}
-
-	if (get_read_path(path, sizeof(path), prefix, name) == 0) {
-		pw_log_debug(NAME" %p: can't load config '%s': %m", conf, path);
-		return -ENOENT;
-	}
 	if ((fd = open(path,  O_CLOEXEC | O_RDONLY)) < 0)  {
 		pw_log_warn(NAME" %p: error loading config '%s': %m", conf, path);
 		return -errno;
@@ -264,42 +343,69 @@ error_close:
 SPA_EXPORT
 int pw_conf_load_conf(const char *prefix, const char *name, struct pw_properties *conf)
 {
-	return conf_load(prefix, name, conf);
+	char path[PATH_MAX];
+
+	if (name == NULL) {
+		pw_log_debug(NAME" %p: config name must not be NULL", conf);
+		return -EINVAL;
+	}
+
+	if (get_config_path(path, sizeof(path), prefix, name) == 0) {
+		pw_log_debug(NAME" %p: can't load config '%s': %m", conf, path);
+		return -ENOENT;
+	}
+
+	return conf_load(path, conf);
 }
 
 SPA_EXPORT
 int pw_conf_load_state(const char *prefix, const char *name, struct pw_properties *conf)
 {
-	return conf_load(prefix, name, conf);
+	char path[PATH_MAX];
+
+	if (name == NULL) {
+		pw_log_debug(NAME" %p: config name must not be NULL", conf);
+		return -EINVAL;
+	}
+
+	if (get_state_path(path, sizeof(path), prefix, name) == 0) {
+		pw_log_debug(NAME" %p: can't load config '%s': %m", conf, path);
+		return -ENOENT;
+	}
+
+	return conf_load(path, conf);
 }
 
-static int parse_spa_libs(struct pw_context *context, const char *str)
+/* context.spa-libs = {
+ *  <factory-name regex> = <library-name>
+ * }
+ */
+static int parse_spa_libs(struct pw_context *context, char *str)
 {
 	struct spa_json it[2];
 	char key[512], value[512];
+	int count = 0;
 
 	spa_json_init(&it[0], str, strlen(str));
-	if (spa_json_enter_object(&it[0], &it[1]) < 0)
+	if (spa_json_enter_object(&it[0], &it[1]) < 0) {
+		pw_log_error("config file error: context.spa-libs is not an object");
 		return -EINVAL;
+	}
 
 	while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
-		const char *val;
-		if (key[0] == '#') {
-			if (spa_json_next(&it[1], &val) <= 0)
-				break;
-		}
-		else if (spa_json_get_string(&it[1], value, sizeof(value)-1) > 0) {
+		if (spa_json_get_string(&it[1], value, sizeof(value)-1) > 0) {
 			pw_context_add_spa_lib(context, key, value);
+			count++;
 		}
 	}
-	return 0;
+	return count;
 }
 
 static int load_module(struct pw_context *context, const char *key, const char *args, const char *flags)
 {
 	if (pw_context_load_module(context, key, args, NULL) == NULL) {
 		if (errno == ENOENT && flags && strstr(flags, "ifexists") != NULL) {
-			pw_log_debug(NAME" %p: skipping unavailable module %s",
+			pw_log_info(NAME" %p: skipping unavailable module %s",
 					context, key);
 		} else if (flags == NULL || strstr(flags, "nofail") == NULL) {
 			pw_log_error(NAME" %p: could not load mandatory module \"%s\": %m",
@@ -309,65 +415,65 @@ static int load_module(struct pw_context *context, const char *key, const char *
 			pw_log_info(NAME" %p: could not load optional module \"%s\": %m",
 					context, key);
 		}
+	} else {
+		pw_log_info(NAME" %p: loaded module %s", context, key);
 	}
 	return 0;
 }
 
-static int parse_modules(struct pw_context *context, const char *str)
+/*
+ * context.modules = [
+ *   {   name = <module-name>
+ *       [ args = { <key> = <value> ... } ]
+ *       [ flags = [ [ ifexists ] [ nofail ] ]
+ *   }
+ * ]
+ */
+static int parse_modules(struct pw_context *context, char *str)
 {
 	struct spa_json it[3];
 	char key[512];
-	int res = 0;
+	int res = 0, count = 0;
 
 	spa_json_init(&it[0], str, strlen(str));
-	if (spa_json_enter_object(&it[0], &it[1]) < 0)
+	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+		pw_log_error("config file error: context.modules is not an array");
 		return -EINVAL;
+	}
 
-	while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
-		const char *val, *aval;
-		char *args = NULL, *flags = NULL;
-		int len, alen;
+	while (spa_json_enter_object(&it[1], &it[2]) > 0) {
+		char *name = NULL, *args = NULL, *flags = NULL;
 
-		if ((len = spa_json_next(&it[1], &val)) <= 0)
-			break;
+		while (spa_json_get_string(&it[2], key, sizeof(key)-1) > 0) {
+			const char *val;
+			int len;
 
-		if (key[0] == '#')
-			continue;
+			if ((len = spa_json_next(&it[2], &val)) <= 0)
+				break;
 
-		if (spa_json_is_object(val, len)) {
-			char arg[512];
+			if (spa_streq(key, "name")) {
+				name = (char*)val;
+				spa_json_parse_string(val, len, name);
+			} else if (spa_streq(key, "args")) {
+				if (spa_json_is_container(val, len))
+					len = spa_json_container_len(&it[2], val, len);
 
-			spa_json_enter(&it[1], &it[2]);
-
-			while (spa_json_get_string(&it[2], arg, sizeof(arg)-1) > 0) {
-				if ((alen = spa_json_next(&it[2], &aval)) <= 0)
-					break;
-
-				if (strcmp(arg, "args") == 0) {
-					if (spa_json_is_container(aval, alen))
-						alen = spa_json_container_len(&it[2], aval, alen);
-
-					args = malloc(alen + 1);
-					spa_json_parse_string(aval, alen, args);
-				} else if (strcmp(arg, "flags") == 0) {
-					if (spa_json_is_container(aval, alen))
-						alen = spa_json_container_len(&it[2], aval, alen);
-
-					flags = malloc(alen + 1);
-					spa_json_parse_string(aval, alen, flags);
-				}
+				args = (char*)val;
+				spa_json_parse_string(val, len, args);
+			} else if (spa_streq(key, "flags")) {
+				if (spa_json_is_container(val, len))
+					len = spa_json_container_len(&it[2], val, len);
+				flags = (char*)val;
+				spa_json_parse_string(val, len, flags);
 			}
 		}
-		else if (!spa_json_is_null(val, len))
-			break;
-
-		res = load_module(context, key, args, flags);
-
-		free(args);
-		free(flags);
+		if (name != NULL)
+			res = load_module(context, name, args, flags);
 
 		if (res < 0)
 			break;
+
+		res = ++count;
 	}
 	return res;
 }
@@ -399,57 +505,59 @@ static int create_object(struct pw_context *context, const char *key, const char
 	return 0;
 }
 
-static int parse_objects(struct pw_context *context, const char *str)
+/*
+ * context.objects = [
+ *   {   factory = <factory-name>
+ *       [ args  = { <key> = <value> ... } ]
+ *       [ flags = [ [ nofail ] ] ]
+ *   }
+ * ]
+ */
+static int parse_objects(struct pw_context *context, char *str)
 {
 	struct spa_json it[3];
 	char key[512];
-	int res = 0;
+	int res = 0, count = 0;
 
 	spa_json_init(&it[0], str, strlen(str));
-	if (spa_json_enter_object(&it[0], &it[1]) < 0)
+	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+		pw_log_error("config file error: context.objects is not an array");
 		return -EINVAL;
+	}
 
-	while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
-		const char *val, *aval;
-		char *args = NULL, *flags = NULL;
-		int len, alen;
+	while (spa_json_enter_object(&it[1], &it[2]) > 0) {
+		char *factory = NULL, *args = NULL, *flags = NULL;
 
-		if ((len = spa_json_next(&it[1], &val)) <= 0)
-			break;
+		while (spa_json_get_string(&it[2], key, sizeof(key)-1) > 0) {
+			const char *val;
+			int len;
 
-		if (key[0] == '#')
-			continue;
+			if ((len = spa_json_next(&it[2], &val)) <= 0)
+				break;
 
-		if (spa_json_is_object(val, len)) {
-			char arg[512];
+			if (spa_streq(key, "factory")) {
+				factory = (char*)val;
+				spa_json_parse_string(val, len, factory);
+			} else if (spa_streq(key, "args")) {
+				if (spa_json_is_container(val, len))
+					len = spa_json_container_len(&it[2], val, len);
 
-			spa_json_enter(&it[1], &it[2]);
+				args = (char*)val;
+				spa_json_parse_string(val, len, args);
+			} else if (spa_streq(key, "flags")) {
+				if (spa_json_is_container(val, len))
+					len = spa_json_container_len(&it[2], val, len);
 
-			while (spa_json_get_string(&it[2], arg, sizeof(arg)-1) > 0) {
-				if ((alen = spa_json_next(&it[2], &aval)) <= 0)
-					break;
-
-				if (strcmp(arg, "args") == 0) {
-					if (spa_json_is_container(aval, alen))
-						alen = spa_json_container_len(&it[2], aval, alen);
-
-					args = malloc(alen + 1);
-					spa_json_parse_string(aval, alen, args);
-				} else if (strcmp(arg, "flags") == 0) {
-					flags = strndup(aval, alen);
-				}
+				flags = (char*)val;
+				spa_json_parse_string(val, len, flags);
 			}
 		}
-		else if (!spa_json_is_null(val, len))
-			break;
-
-		res = create_object(context, key, args, flags);
-
-		free(args);
-		free(flags);
+		if (factory != NULL)
+			res = create_object(context, factory, args, flags);
 
 		if (res < 0)
 			break;
+		res = ++count;
 	}
 	return res;
 }
@@ -478,56 +586,57 @@ static int do_exec(struct pw_context *context, const char *key, const char *args
 		}
 	}
 	else {
-		int status;
+		int status = 0;
 		res = waitpid(pid, &status, WNOHANG);
 		pw_log_info("exec got pid %d res:%d status:%d", pid, res, status);
 	}
 	return 0;
 }
 
-static int parse_exec(struct pw_context *context, const char *str)
+/*
+ * context.exec = [
+ *   { path = <program-name>
+ *     [ args = "<arguments>" ]
+ *   }
+ * ]
+ */
+static int parse_exec(struct pw_context *context, char *str)
 {
 	struct spa_json it[3];
 	char key[512];
-	int res = 0;
+	int res = 0, count = 0;
 
 	spa_json_init(&it[0], str, strlen(str));
-	if (spa_json_enter_object(&it[0], &it[1]) < 0)
+	if (spa_json_enter_array(&it[0], &it[1]) < 0) {
+		pw_log_error("config file error: context.exec is not an array");
 		return -EINVAL;
+	}
 
-	while (spa_json_get_string(&it[1], key, sizeof(key)-1) > 0) {
-		const char *val;
-		char *args = NULL;
-		int len;
+	while (spa_json_enter_object(&it[1], &it[2]) > 0) {
+		char *path = NULL, *args = NULL;
 
-		if ((len = spa_json_next(&it[1], &val)) <= 0)
-			break;
+		while (spa_json_get_string(&it[2], key, sizeof(key)-1) > 0) {
+			const char *val;
+			int len;
 
-		if (key[0] == '#')
-			continue;
+			if ((len = spa_json_next(&it[2], &val)) <= 0)
+				break;
 
-		if (spa_json_is_object(val, len)) {
-			char arg[512], aval[1024];
-
-			spa_json_enter(&it[1], &it[2]);
-
-			while (spa_json_get_string(&it[2], arg, sizeof(arg)-1) > 0) {
-				if (spa_json_get_string(&it[2], aval, sizeof(aval)-1) <= 0)
-					break;
-
-				if (strcmp(arg, "args") == 0)
-					args = strdup(aval);
+			if (spa_streq(key, "path")) {
+				path = (char*)val;
+				spa_json_parse_string(val, len, path);
+			} else if (spa_streq(key, "args")) {
+				args = (char*)val;
+				spa_json_parse_string(val, len, args);
 			}
 		}
-		else if (!spa_json_is_null(val, len))
-			break;
-
-		res = do_exec(context, key, args);
-
-		free(args);
+		if (path != NULL)
+			res = do_exec(context, path, args);
 
 		if (res < 0)
 			break;
+
+		res = ++count;
 	}
 	return res;
 }
@@ -537,21 +646,26 @@ int pw_context_parse_conf_section(struct pw_context *context,
 		struct pw_properties *conf, const char *section)
 {
 	const char *str;
+	char *s;
 	int res;
 
 	if ((str = pw_properties_get(conf, section)) == NULL)
-		return -ENOENT;
+		return 0;
 
-	if (strcmp(section, "context.spa-libs") == 0)
-		res = parse_spa_libs(context, str);
-	else if (strcmp(section, "context.modules") == 0)
-		res = parse_modules(context, str);
-	else if (strcmp(section, "context.objects") == 0)
-		res = parse_objects(context, str);
-	else if (strcmp(section, "context.exec") == 0)
-		res = parse_exec(context, str);
+	s = strdup(str);
+
+	if (spa_streq(section, "context.spa-libs"))
+		res = parse_spa_libs(context, s);
+	else if (spa_streq(section, "context.modules"))
+		res = parse_modules(context, s);
+	else if (spa_streq(section, "context.objects"))
+		res = parse_objects(context, s);
+	else if (spa_streq(section, "context.exec"))
+		res = parse_exec(context, s);
 	else
 		res = -EINVAL;
 
+	free(s);
+
 	return res;
 }
